@@ -1,3 +1,6 @@
+''' Alapszintű helper függvények:  stingkezelés (pl. text2int, d_annotate, d_lookup), tömbök és szótárak kezelése, fájlrendszer, debug
+''' 
+
 from time import process_time,perf_counter
 import os
 from itertools import groupby
@@ -36,6 +39,7 @@ def kodto(kod,kodlista):
     '''
     if not kod: return ''
     if not kodlista: return ''
+    if type(kod)!=str: kod=str(kod) 
     kereső=kod+':'
     nLen=len(kereső)
     aSor=kodlista.split('//')
@@ -46,6 +50,7 @@ def kodto(kod,kodlista):
 
 def beginwith(strIn,samples,replace=None):   # -> found_sample vagy ''     replace esetén a cserével létrejövő string
     ''' samples:  minták | jeles felsorolása   példa:  'ki|be'
+      FIGYELEM:  pont, zárójel és egyéb regex karakterek elé kötelezően '\' kell   (pl. r'\.')
     return:  üres vagy a talált minta
     process_time: mikrosec körül
     '''
@@ -84,8 +89,9 @@ def vanbenne(strIn,samples):
     if m==None: return ''
     else: return m.group()
 
-
-
+def cutleft(strIn,strCut):
+    # Nem ellenőrzi, hogy valóban a strCut-tal kezdődik-e, egyszerűen levágja az elejéről a strCut hosszának megfelelő karaktereket
+    return strIn[len(strCut):]
 
 def trim(str):
     ''' strip a széleken + ismétlődő space-k törlése belül 
@@ -118,6 +124,19 @@ def clean(str,accentlevel='soft',bDropSpaces=False):
         str=' '.join(str.split())
         return str.lower()
 
+def skipaccents(str,level='soft'):
+    ''' Ékezetek elhagyása
+    level:
+       'soft':  csak a hosszú ékezeteket cseréli rövidre,  a-á, e-á, o-ö, u-ü eltérés megmarad
+       'hard':  minden ékezetet elhagy
+    '''
+    if level=='soft':
+        return str.translate(str.maketrans('íóőúű','ioöuü'))
+    elif level=='hard':
+        return str.translate(str.maketrans('áéíóöőúüű','aeiooouuu'))
+    else:
+        return str
+
 
 def splitfirst(str,separator=None):
     '''  Hívása:  first,second = splitfirst(str,',') 
@@ -398,90 +417,101 @@ def d_lookup(strIn,d_samples,tupleout=False,bWholesamples=True):
     Ha nincs találat, akkor None
     strIn: általában egyetlen szó   (szavanként, vagy néhány szavas ablakokkal kell hívni a függvényt)
     d_samples:   {[keresőszó]:[value],...}         A keresőszó string, a value lehet szám is vagy akár egy sub dictironary
-       teljes-egyezéses minta: pont a keresőszó végén
-    tupleout: True esetén return (key,value)
+       - a keresőszavakban csak számok és betűk lehetnek
+       - a keresőszavak végén lehet egy pont:  teljes-egyezéses minta   (ha nincs pont, akkor beginwith egyezés is jó)
+       
+    tupleout: True esetén return (key,value).    A key lehet több szavas  (az esetleges lezáró pont nélkül adja vissza a függvény)
     bWholesamples:  vannak-e teljes-egyezéses minták is a d_samples-ben (pont a keresőszó végén) 
     process_time:   2 microsec     50-es dict, átlagos szó (nem talál)     Feltehetően nagy dict-re is gyors (hash index)
     '''
     
     result=None
 
-    keresoszo=strIn
+    tosearch=strIn
 
     if bWholesamples:
-        result=d_samples.get(keresoszo + '.')
-        if result: 
-            if tupleout: return keresoszo,result
-            else: return result
+        tosearch=tosearch.replace(' ','.') + '.'
+        
+        #result=d_samples.get(tosearch + '.')
+        #if result: 
+        #    if tupleout: return tosearch,result
+        #    else: return result
 
     # Keresés hosszrövidítésekel (a leghosszabb keresőminta érvényesül)
-    while len(keresoszo)>1:
-        #print(str(type(d_samples)))
-        result=d_samples.get(keresoszo)
+    while len(tosearch)>1:
+        result=d_samples.get(tosearch)
         if result: 
-            if tupleout: return keresoszo,result
+            if tupleout: 
+                tosearch=tosearch.replace('.',' ')
+                if tosearch[-1]==' ': tosearch=tosearch[:-1]
+                return tosearch,result
             else: return result
-        keresoszo=keresoszo[:-1]
+        tosearch=tosearch[:-1]
 
-    if result:
-        if tupleout: return keresoszo,result
-        else: return result
-    else:
-        if tupleout: return None,None
-        else: return None
+    #if result:
+    #    if tupleout: return tosearch,result
+    #    else: return result
+    #else:
+    if tupleout: return None,None
+    else: return None
 
 
        
 
-def d_annotate(strIn,lookup,bNum=False):    # -> pattern,invalues,outvalues
+def d_annotate(strIn,lookup,bNum=False,max_words_in_samples=1,accentlevel='soft'):    # -> pattern,invalues,outvalues
     ''' Annotálás jellegű művelet. A szöveg mintázatát állítja elő.  
     A beazonosítható szövegrészek helyébe szögletes zárójelben az entity-nevét írja, az eredeti és a szabványosított értéket pedig 
         menti az invalues és outvalues tömbbe (a két tömb indexelése a pattern-be kerülő "[...]" helyettesítőjelekhez igazodik) 
 
-    strIn:  általában több szavas, lower és trim előkészítéssel  (használható pl a clean() helper)
-    lookup:   {'[startwords]':'[entity],[value]', ...}
-        - [startwords]: több szavas is lehet | határolással.  Példa:  'múlt|elmúlt|előző|legutóbbi'
+    strIn:  általában több szavas kifejezés vagy mondat.    
+        Nem kell előzetesen szabványosítani (a jelen függvényben: lower, trim, skipaccents, skippunctuations)
+    lookup:   {'[startsamples]':'[entity],[value]', ...}
+        - [startsamples]: több keresőminta is megadható | határolással.  Példa:  'múlt|elmúlt|előző|legutóbbi'
                 A sorrend érdektelen, mindig a lehetséges leghosszabb találat érvényesül ("tavasszal" akkor is talál, ha a "tavasz" minta előrébb áll)
-        - [entity]:  entity-név (pl. "évszak").  Előfordulhat, hogy ugyanaz az entity többször is előfordul a mondatban
-                Ugyanaz az entitás-név több startwords-höz is megadható.  Tetszőleges számú entitásnév lehet a d_regex-ben.
+                A keresőminták több szavasak is lehetnek (trimmelve kell megadni)
+                Ha egy keresőminta végén pont van, akkor csak wholeword találat megengedett  (egyébként beginwith találat is jó)
+        - [entity]:  entity-név (pl. "évszak"). Kötelezően egyszavas  ('_' lehet benne)  
+                Ugyanaz az entitás-név több startwords-höz is megadható.  Tetszőleges számú entitásnév lehet a lookup-ben.
                 "none" entitást kell megadni, ha csak szaványosításról illetve szinonimák összevonásáról van szó
-                  Példa:  "korábbi|előző":none,korábbi.     Ebben az esetben csere a szabványosított értékre, a d_values-be nem kerül bele
+                  Példa:  "korábbi|előző":none,korábbi.     Ebben az esetben csere a szabványosított értékre, az outvalues-be nem kerül bele
                 "stopword" entitás-névvel kell felsorolni az elhagyandó szavakat (pl. névelők)
-        - [value]:  ez az érték kerül be a d_values-be (az adott entity névvel). Szabványosított / kivonatolt érték
-                Ha nincs megadva, akkor megegyezik a startword-del
-                Lehet szám, dátum, string is
-    bNum:  True esetén a szöveges vagy numerikus szám-szavak annotálása [szám] entity-névvel (sorszámok és római számok is)
-    
+        - [value]:  ez az érték kerül be az outvalues-be (az adott entity névvel). Szabványosított / kivonatolt érték
+                Ha nincs megadva, akkor a talált startsample lesz a szabványosított érték
+                Lehet szám, dátum, string is.  String esetén több szavas is lehet (trimmelve kell megadni)
+    bNum:  True esetén a szöveges vagy numerikus szám-szavak annotálása "[szám]" entity-névvel (sorszámok és római számok is)
+    max_words_in_samples:  maximum hány szavas minták vannak a lookup-ben
+        - ha kisebb szám van megadva, akkor az ennél hosszabb minták nem fognak találatot adni 
+    accentlevel:     '': nincs összevonás,  'soft':  a-á, e-é, o-ö, u-ü nincs összevonva     'hard':  erős összevonás
+      
+
     return   pattern,invalues,outvalues
-                pattern: "[entity1] egyébszó [entity2]"
-                invalues:  az annotált részek eredeti értéke  (a fenti példához 2-elemű lista tartozik)
-                outvalues:  az annotált részek kimeneti értéke  (a fenti példához 2-elemű lista tartozik)
+                pattern:    példa:  "[entity1] egyébszó [entity2]"
+                invalues:   az annotált részek eredeti értéke  (a fenti pattern-példához 2-elemű lista tartozik)
+                outvalues:  az annotált részek kimeneti értéke  (a fenti pattern-példához 2-elemű lista tartozik)
     '''
 
     invalues=[]
     outvalues=[]
 
-    # key-felsorolások kibontása
+    dropchars=string.punctuation.replace('.','')        # a '.' karakteren kívüli összes írásjel
+
+    # key-felsorolások kibontása (ha nincs még kibontva;  a lookup egy globális objektum, amit egyszer kell csak kibontani)
     if not lookup.get('d_compiled'):
-        dict_compiled={'d_compiled':True}
+        dict_compiled={'d_compiled':True}     # a d_compiled adat helyből beállítva
         for key,value in lookup.items():
-            subkeys=key.split('|')
+            subkeys=key.split('|')          # több keresőminta is felsorolható '|' határolással
             for subkey in subkeys: 
-                if subkey: dict_compiled[subkey]=value
+                if subkey: 
+                    subkey=subkey.lower()
+                    subkey=skipaccents(subkey,accentlevel)
+                    # írásjelek helyett szóközök (kivéve '.')
+                    #strL=strL.translate(str.maketrans(dropchars,' '*len(dropchars)))
+
+                    subkey=subkey.replace(' ','.')       # a több szavas keresőmintákban "." kell a szóközök helyett (technikai oka van)
+                    dict_compiled[subkey]=value
         lookup=dict_compiled
     
 
-    # A VÁLTOZAT:  szavanként keresés a mintákat tartalmazó dict-ben  (like: karakter-elhagyások a végéről)
-    # probléma:  lehetnek teljes szavas keresőminták is
-    #  megoldás:  a teljes szavas mintákat el kell különíteni
-    #     d_like   d_wholeword
-    # VAGY listák:
-    #   a dict előfeldolgozása keretében külön rekordba sorolandók
-    #   "korábbi*|előző*:none,korábbi
-    #     korábbi*:none,korábbi
-    #     előző*:none,korábbi
-
-    
     dropchars=string.punctuation.replace('.','')        # a '.' karakteren kívüli összes írásjel
 
     strL=strIn
@@ -491,40 +521,113 @@ def d_annotate(strIn,lookup,bNum=False):    # -> pattern,invalues,outvalues
     strL=strL.replace('.','. ')
     
     strL=strL.lower()
-    # str=str.translate(str.maketrans('íóőúű','ioöuü'))
+    strL=skipaccents(strL,accentlevel)
     
     # trim
     words=strL.split()
 
-    
+    # magában álló pontok elhagyása (nem volt előtte alfanum, érdektelen)
     wordsout=[]
     for word in words:
-        if word=='.': continue   # a . előtt nem volt alfanum (érdektelen)
-
+        if word=='.': continue
+        wordsout.append(word)
+    words=wordsout
+    
+    # Keresés szavanként (vagy pár szavas csúszóablakkal) a mintákat tartalmazó dict-ben  (like: karakter-elhagyások a végéről)
+    wordsout=[]
+    nLen=len(words)
+    i=0
+    while i<nLen:
         if bNum:
-            if not (word=='hét' and len(wordsout)>0):   # a "hét" szó helyébe ne írjon számot, ha nem az első szó (összetéveszthető a "hét" időhatározóval)
+            word=words[i]     # a számoknál csak egyszavas keresés van
+            if word!='hét':   # a "hét" szó helyébe ne írjon számot (összetéveszthető a "hét" időhatározóval)
+                              # Később még ellenőrzöm, hogy ha [időtartam] a következő szó is, akkor [szám]-ra módosuljon az értelmezése (pl. "hét nap")
                 n=text2int(word,True,False)
                 if n>-1:
                     invalues.append(word)
                     outvalues.append(n)
                     wordsout.append('[szám]')
+                    i+=1
                     continue
 
-        key,foundvalue=d_lookup(word,lookup,True)
+        # max_words_in_samples szóra kell keresni (ha van még ennyi szó)
+        i2 = i + max_words_in_samples
+        if i2>nLen: i2=nLen
+        tosearch=' '.join(words[i:i2])
+        
+        key,foundvalue=d_lookup(tosearch,lookup,True)
         if foundvalue:
+            # hány szavas a találat (ennyi szó lesz lecserélve a helyettesítőjellel)
+            nFoundWords=len(key.split())
+            
             entity,value=splitfirst(foundvalue,',')
             if entity=='stopword': wordout=''
             else:
                 if value=='': value=key
                 if entity!='none': 
+                    # Ha az előző szó is [időtartam] volt "hét" input-értékkel, akkor előző szó legyen számként annotálva (pl. "hét nap" "[szám] [időtartam]"
+                    if entity=='időtartam' and len(wordsout)>0 and wordsout[-1]=='[időtartam]' and invalues[-1]=='hét':
+                        wordsout[-1]='[szám]'
+                        outvalues[-1]=7
                     if entity=='szám': value=int(value)
-                    invalues.append(word)
+                    invalues.append(' '.join(words[i:i+nFoundWords]))
                     outvalues.append(value)
                     wordout='[' + entity + ']'
+                # Ha a keresőmintához "none" lett megadva entitásként, akkor nem helyettesítőjellel, hanem a szabványosított értékkel
+                #    kell lecserélni a találatot eredményező szövegrészt  (a value több szavas is lehet) 
                 else:
                     wordout=value
-        else: wordout=word
+            i += nFoundWords
+
+        else: 
+            # Ha nincs találat, akkor egyetlen szó kerül be a wordsout-ba (változtatás nélkül), majd lépés a következő szóra
+            wordout=words[i]
+            i+=1
+
+
+
         if wordout!='': wordsout.append(wordout)
+    
+
+
+
+    #for word in words:
+    #    if word=='.': continue   # a . előtt nem volt alfanum (érdektelen)
+
+    #    if bNum:
+    #        if word!='hét':   # a "hét" szó helyébe ne írjon számot (összetéveszthető a "hét" időhatározóval)
+    #                          # Később még ellenőrzöm, hogy ha [időtartam] a következő szó is, akkor [szám]-ra módosuljon az értelmezése (pl. "hét nap")
+    #            n=text2int(word,True,False)
+    #            if n>-1:
+    #                invalues.append(word)
+    #                outvalues.append(n)
+    #                wordsout.append('[szám]')
+    #                continue
+
+    #    key,foundvalue=d_lookup(word,lookup,True)
+    #    if foundvalue:
+    #        entity,value=splitfirst(foundvalue,',')
+    #        if entity=='stopword': wordout=''
+    #        else:
+    #            if value=='': value=key
+    #            if entity!='none': 
+    #                # Ha az előző szó is [időtartam] volt "hét" input-értékkel, akkor előző szó legyen számként annotálva (pl. "hét nap" "[szám] [időtartam]"
+    #                if entity=='időtartam' and len(wordsout)>0 and wordsout[-1]=='[időtartam]' and invalues[-1]=='hét':
+    #                    wordsout[-1]='[szám]'
+    #                    outvalues[-1]=7
+    #                if entity=='szám': value=int(value)
+    #                invalues.append(word)
+    #                outvalues.append(value)
+    #                wordout='[' + entity + ']'
+    #            else:
+    #                wordout=value
+    #    else: wordout=word
+
+
+
+    #    if wordout!='': wordsout.append(wordout)
+    
+        
     pattern=' '.join(wordsout)
     return pattern,invalues,outvalues
 
